# Path from where to find files
INPUT_PATH = "./input"

# Folder inside input that holds non-template files
CONTENT_DIR_NAME = "content"

# Folder in which to output files
OUTPUT_PATH = "./output"

# List of css files to bundle, order counts, use None for all css files in css folder
CSS_FILES = None

# Removes current output folder before generating
CLEAN_OUTPUT = True

# For log messages
import logging

LOG_FORMAT = "[%(levelname)s] %(message)s"
LOG_LEVEL = logging.DEBUG

# To enable escaping by default on Jinja2 prints
AUTOESCAPE = False

# Seconds to wait before scanning files for new changes
WATCH_WAIT_TIME = 0.5



import json
import logging
import os
from pathlib import Path

import yaml

import sssimp
from sssimp import jinja
from sssimp.utils import path_strip

from .markdown import MarkdownPage

DATA_DIR = sssimp.INPUT_DIR / "data"


class Data:
    def __init__(self, base_path=DATA_DIR):
        self.dict = {}
        self.flat = {}
        self.base_path = base_path

    def handle_file(self, path: Path):
        def opener(func):
            def wrapped(path):
                with path.open() as f:
                    return func(f)

            return wrapped

        parents = [
            x.split(".")[0]
            for x in path_strip(path, self.base_path).split(os.path.sep)
        ]
        getter = {
            "md": read_markdown,
            "json": opener(json.load),
            "yaml": opener(yaml.safe_load),
            "yml": opener(yaml.safe_load),
        }.get(path.suffix[1:])
        if not getter:
            logging.warning(f"Unsupported data file {path}")
            return
        data = getter(path)
        logging.debug(f"Data: {path} -> {data}")
        target = self.dict
        for parent in parents[:-1]:
            target = target.setdefault(parent, {})
        target[parents[-1]] = data
        self.flat[path.stem] = data


def prepare():
    data = get(DATA_DIR)
    jinja.globals["data"] = data


def get(path: str, flat=False):
    path = Path(path)
    data = Data(base_path=path)
    for file in path.rglob("*.*"):
        logging.info(f"Handling data file {file}")
        data.handle_file(file)
    if flat:
        return data.flat
    return data.dict


def read_markdown(path: Path):
    logging.info("read_markdown")
    return MarkdownPage(path, path)



import functools
import logging
from datetime import datetime
from pathlib import Path

import sssimp
from sssimp import jinja
from sssimp.utils import mkdir, path_strip

PAGES = set()


class Page:
    def __init__(self, src, target):
        self.src = src
        self.target = target
        self.vars = {
            "page": self,
        }
        self.meta = None

    def __str__(self):
        return str(self.target)

    @property
    def created_at(self):
        return datetime.fromtimestamp(self.stat.st_ctime)

    @property
    def updated_at(self):
        time = datetime.fromtimestamp(self.stat.st_mtime)
        if time == self.created_at:
            return None
        return time

    @property
    # TODO 3.9+: @functools.cache
    # 3.8 compat:
    @functools.lru_cache()
    def stat(self):
        return self.src.stat()

    @property
    def href(self):
        return "/".join(Path(path_strip(self.target, sssimp.OUTPUT_DIR)).parts)

    @property
    def name(self):
        return self.target.name

    @property
    def parent(self):
        return self.target.parent

    def get_template(self):
        with open(self.src) as f:
            content = f.read()
        return jinja.from_string(content)

    # TODO 3.9+: @functools.cache
    # 3.8 compat:
    @functools.lru_cache()
    def render(self):
        return self.get_template().render(**self.vars) + "\n"

    def write(self, target=None):
        target = target or self.target
        mkdir(self.target)
        with open(self.target, "w") as f:
            logging.info(f"Generating {self.target}")
            f.write(self.render())


def prepare():
    for file in sssimp.CONTENT_DIR.glob("**/*.html"):
        target = sssimp.OUTPUT_DIR / path_strip(file, sssimp.CONTENT_DIR)
        PAGES.add(Page(src=file, target=target))
        sssimp.IGNORE_ASSETS.add(str(file))


def main():
    jinja.globals["pages"] = PAGES
    for page in PAGES:
        page.write()



import difflib
import filecmp
import os
import shutil
import sys
from pathlib import Path

import pytest

root = Path(".")
examples = root.glob("examples/*/")


def check_diffs(diff, example):
    assert diff.left_only == []
    assert diff.right_only == []
    success = True
    for filename in diff.diff_files:
        success = False
        print(f"File with difference: {example / filename}")
        left_content = (
            (Path(diff.left) / filename).read_text().splitlines(keepends=True)
        )
        right_content = (
            (Path(diff.right) / filename).read_text().splitlines(keepends=True)
        )
        for row in difflib.unified_diff(left_content, right_content):
            print(row, end="")
        print()

    for sub_dcmp in diff.subdirs.values():
        success = check_diffs(sub_dcmp, example) and success
    return success


@pytest.mark.parametrize("example", examples)
def test_examples(example, tmpdir, request):
    outdir = tmpdir / "output"
    cmd = f"sssimp --input {example / 'input'} {outdir}"
    print(cmd)
    exit_code = os.system(cmd)
    assert exit_code == 0
    expected_output = root / example / "output"
    save = request.config.getoption("--save")
    if save:
        if expected_output.exists():
            shutil.rmtree(expected_output)
        shutil.move(outdir, expected_output)
        return

    diff = filecmp.dircmp(expected_output, outdir)
    print(expected_output, outdir)
    assert check_diffs(
        diff,
        example,
    ), "Some files differ. See above using the -s flag of pytest"



import logging

import sssimp
from sssimp import config, jinja
from sssimp.utils import mkdir, path_strip

BUNDLE_FILE = "bundle.css"
OUT_FILE = sssimp.OUTPUT_DIR / BUNDLE_FILE
CSS_DIR = sssimp.INPUT_DIR / "css"
css_files = None


def prepare():
    global css_files
    jinja.globals["BUNDLE_FILE"] = BUNDLE_FILE
    css_files = list(
        [CSS_DIR / file for file in config.CSS_FILES]
        if config.CSS_FILES
        else CSS_DIR.glob("**/*.css")
    )
    if css_files:
        jinja.globals["BUNDLE_TIME"] = max(x.stat().st_ctime for x in css_files)


def main():
    if not css_files:
        logging.debug("No CSS files found, skipping")
        return
    mkdir(OUT_FILE)
    with open(OUT_FILE, "w") as out:
        for file in css_files:
            relative_path = path_strip(file, CSS_DIR)
            logging.info(f"Bundling {relative_path}")
            print(f"/** bundle:{relative_path} **/", file=out)
            with open(file) as css_file:
                out.write(css_file.read())
            print(file=out)



import logging

import sssimp
from sssimp.utils import path_strip

FILES = set()


def prepare():
    for file in sssimp.CONTENT_DIR.glob("**/*.j2"):
        logging.info(f"Handling Jinja2 template {file}")
        FILES.add(file)
        sssimp.IGNORE_ASSETS.add(str(file))


def main():
    for file in FILES:
        target = (
            sssimp.OUTPUT_DIR / path_strip(file, sssimp.CONTENT_DIR)
        ).with_suffix("")
        with file.open() as f:
            content = f.read()
        templated = sssimp.jinja.from_string(content)
        target.parent.mkdir(parents=True, exist_ok=True)
        with target.open("w") as f:
            f.write(templated.render() + "\n")
